!{\src2tex{textfont=tt}}
!!****f* ABINIT/pawbcast
!! NAME
!! pawbcast
!!
!! FUNCTION
!! Communicate paw data to all processors
!!
!! COPYRIGHT
!! Copyright (C) 2009-2013 ABINIT group (FJ,MT)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt.
!!
!! INPUTS
!! comm_mpi= communicator used to broadcast data
!! lmnmax= max number of (l,m,n) comp. over all type of psps
!! lnmax= Max. number of (l,n) components over all type of psps
!! mqgrid_ff= dimension of ffspl
!! mqgrid_vl= dimension of vlspl
!!
!! SIDE EFFECTS
!!  epsatm=$ (4\pi)\int_0^\infty [r^2 (V(r)+\frac{Zv}{r}) dr]$(hartree)
!!  ffspl(mqgrid_ff,2,lnmax)=Kleinman-Bylander form factor f_l(q) and
!!  indlmn(6,i)= array giving l,m,n,lm,ln,s for i=ln  (if useylm=0)
!!   second derivative from spline fit for each angular momentum and
!!   each projector; if any, spin-orbit components begin at l=mpsang+1 or i=lmn (if useylm=1)
!!  paw_atdata=<type_paw_atdata_type>
!!  pawrad=<type pawrad_type>
!!  pawtab=<type pawtab_type>
!!   all the important information from the
!!   pseudopotential file. On one processor at input,
!!   on all processors at output
!!  pspso(INOUT)=spin-orbit characteristics, govern the content of ffspl and ekb
!!  vlspl(mqgrid_vl,2)=q^2 Vloc(q) and second derivatives from spline fit
!!  xcccrc=XC core correction cutoff radius (bohr) from psp file
!! PARENTS
!!      pspatm_abinit,pspatm_pspio
!!
!! CHILDREN
!!      paw_atdata_bcast,pawrad_bcast,pawtab_bcast,timab,xcast_mpi
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

#include "abi_common.h"


subroutine pawbcast(comm_mpi,epsatm,ffspl,indlmn,lmnmax,lnmax,mqgrid_ff,mqgrid_vl,paw_atdata,pawrad,pawtab,pspso,vlspl,xcccrc)

 use defs_basis
 use defs_abitypes
 use m_pawrad, only : pawrad_type, pawrad_bcast
 use m_pawtab, only : pawtab_type, pawtab_bcast
 use m_xmpi
 use m_profiling
#if defined HAVE_MPI2
 use mpi
#endif

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'pawbcast'
 use interfaces_18_timing
 use interfaces_51_manage_mpi, except_this_one => pawbcast
!End of the abilint section

 implicit none

#if defined HAVE_MPI1
 include 'mpif.h'
#endif

!Arguments ------------------------------------
 integer,intent(in) :: comm_mpi,lmnmax,lnmax,mqgrid_ff,mqgrid_vl
 integer,intent(inout) :: pspso
 real(dp),intent(inout) :: epsatm,xcccrc
 integer,intent(inout) :: indlmn(6,lmnmax)
 real(dp),intent(inout) :: ffspl(mqgrid_ff,2,lnmax)
 real(dp),intent(inout) :: vlspl(mqgrid_vl,2)
 type(paw_atdata_type),intent(inout) :: paw_atdata
 type(pawrad_type),intent(inout) :: pawrad
 type(pawtab_type),intent(inout) :: pawtab

!Local variables-------------------------------
 integer :: ierr,indx,me,nn,nsz,isz1,isz2
 integer,allocatable :: list_int(:)
 real(dp) :: tsec(2)
 real(dp),allocatable :: list_dpr(:)

!*************************************************************************

 call timab(48,1,tsec)
!call leave_test()

!Broadcast pawrad
 call pawrad_bcast(pawrad,comm_mpi)

!Broadcast pawtab
 call pawtab_bcast(pawtab,comm_mpi)

!Broadcast paw_atdata
 call paw_atdata_bcast(paw_atdata,comm_mpi)

!Brodcast the integers
 me=xcomm_rank(comm_mpi)
 if(me==0) then
   nn=2+size(ffspl)+size(vlspl)
 end if
 nsz=2+size(indlmn)

 ABI_ALLOCATE(list_int,(nsz))
 if(me==0) then
   list_int(1)=pspso
   list_int(2)=nn
   isz1=size(indlmn)
   list_int(3:3+isz1-1)=reshape(indlmn(:,:),(/isz1/))
 end if

 call xcast_mpi(list_int,0,comm_mpi,ierr)

 if(me/=0) then
   pspso =list_int(1)
   nn =list_int(2)
   isz1=lmnmax
   indlmn(:,:)=reshape(list_int(3:3+6*isz1-1),(/6,isz1/))
 end if
 ABI_DEALLOCATE(list_int)

!Broadcast the reals
 ABI_ALLOCATE(list_dpr,(nn))
 if(me==0) then
   list_dpr(1)=epsatm
   list_dpr(2)=xcccrc
   isz1=size(vlspl)
   list_dpr(3:3+isz1-1)=reshape(vlspl(:,:),(/isz1/))
   isz2=size(ffspl)
   list_dpr(3+isz1:3+isz1+isz2-1)=reshape(ffspl(:,:,:),(/isz2/))
 end if

 call xcast_mpi(list_dpr,0,comm_mpi,ierr)

 if(me/=0) then
   epsatm=list_dpr(1)
   xcccrc=list_dpr(2)
   indx=3
   isz1=mqgrid_vl
   vlspl(:,:)=reshape(list_dpr(indx:indx+2*isz1-1),(/isz1,2/))
   indx=indx+2*isz1
   isz1=mqgrid_ff
   isz2=lnmax
   ffspl(:,:,:)=reshape(list_dpr(indx:indx+2*isz1*isz2-1),(/isz1,2,isz2/))
 end if
 ABI_DEALLOCATE(list_dpr)

 call timab(48,2,tsec)

end subroutine pawbcast
!!***
