!{\src2tex{textfont=tt}}
!!****f* ABINIT/psp17in
!! NAME
!! psp17in
!!
!! FUNCTION
!! Initialize pspcod=17 ("PAW  XML pseudopotentials"):
!! continue to read the corresponding file and compute the form factors
!!
!! COPYRIGHT
!! Copyright (C) 1998-2013 ABINIT group (FJ, MT)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!  ipsp= id in the array of the currently read pseudo.
!!  ixc=exchange-correlation choice from main routine data file
!!  lmax=value of lmax mentioned at the second line of the psp file
!!  lmnmax=if useylm=1, max number of (l,m,n) comp. over all type of psps
!!            =if useylm=0, max number of (l,n)   comp. over all type of psps
!!  lnmax=max. number of (l,n) components over all type of psps
!!            angular momentum of nonlocal pseudopotential
!!  mmax=max number of pts in real space grid (already read in the psp file header)
!!  mqgrid_ff=dimension of q (or G) grid for nl form factors (array ffspl)
!!  mqgrid_vl=dimension of q (or G) grid for Vloc (array vlspl)
!!  pawxcdev=choice of XC development (0=no dev. (use of angular mesh) ; 1 or 2=dev. on moments) 
!!  pspheads= header of the current pseudopotential
!!  pspso=spin-orbit characteristics, govern the content of ffspl and ekb
!!            if =0 : this input requires NO spin-orbit characteristics of the psp
!!            if =2 : this input requires HGH characteristics of the psp
!!            if =3 : this input requires HFN characteristics of the psp
!!            if =1 : this input will be changed at output to 1, 2, 3, according
!!                    to the intrinsic characteristics of the psp file
!!  qgrid_ff(psps%mqgrid_ff)=values of q on grid from 0 to qmax (bohr^-1) for nl form factors
!!  qgrid_vl(psps%mqgrid_vl)=values of q on grid from 0 to qmax (bohr^-1) for Vloc
!!  xclevel= XC functional level
!!  zion=nominal valence of atom as specified in psp file
!!  znucl=atomic number of atom as specified in input file to main routine
!!
!! OUTPUT
!!  epsatm=$ (4\pi)\int_0^\infty [r^2 (V(r)+\frac{Zv}{r}) dr]$(hartree)
!!  ffspl(psps%mqgrid_ff,2,psps%lnmax)=form factor f_l(q) and second derivative
!!   from spline fit for each angular momentum and each projector;
!!   if any, spin-orbit components begin at l=mpsang+1
!!  indlmn(6,psps%lmnmax)= array giving l,m,n,lm,ln,s for i=lmn
!!  pawrad <type(pawrad_type)>=paw radial mesh and related data
!!  pawtab <type(pawtab_type)>=paw tabulated starting data
!!  vlspl(psps%mqgrid_vl,2)=q^2 Vloc(q) and second derivatives from spline fit
!!  wvl_crmult,wvl_frmult= variables definining the fine and coarse grids in a wavelets calculation
!!  xc_denpos= lowest allowed density (usually for the computation of the XC functionals)
!!  xcccrc=XC core correction cutoff radius (bohr) from psp file
!!
!! NOTES
!!  Spin-orbit not yet implemented (to be done)
!!
!! PARENTS
!!      pspatm_abinit
!!
!! CHILDREN
!!      bound_deriv,pawrad_copy,pawrad_destroy,pawrad_destroy_array,pawrad_init
!!      pawvhnzc,psp7calc,shapebes,spline,splint,wrtout
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

#include "abi_common.h"

subroutine psp17in(epsatm,ffspl,indlmn,ipsp,ixc,lmax,lmnmax,lnmax,mmax,mqgrid_ff,mqgrid_vl,&
&                  pawrad,pawtab,pawxcdev,pspheads,&
&                  pspso,qgrid_ff,qgrid_vl,vlspl,xcccrc,xclevel,xc_denpos,zion,znucl)

 use defs_basis
 use defs_datatypes
 use m_errors
 use m_splines
 use m_profiling
 use m_xml_pawpseudo_types
 use m_xml_pawpseudo

 use m_pawtab, only : pawtab_type
 use m_pawrad, only : pawrad_type, pawrad_init, pawrad_destroy, pawrad_destroy_array, &
&                     pawrad_copy, bound_deriv

#if defined HAVE_DFT_LIBXC
  use libxc_functionals
#endif

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'psp17in'
 use interfaces_14_hidewrite
 use interfaces_64_atompaw
 use interfaces_65_psp, except_this_one => psp17in
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: ipsp,ixc,lmax,lmnmax,lnmax,mqgrid_ff,mqgrid_vl,pawxcdev,pspso
 integer,intent(inout) ::mmax
 integer,intent(in) :: xclevel
 real(dp),intent(in) :: xc_denpos,zion,znucl
 real(dp),intent(out) :: epsatm,xcccrc
 type(pawrad_type),intent(out) :: pawrad
 type(pawtab_type),intent(out) :: pawtab
 type(pspheader_type),intent(in)  :: pspheads
 
!arrays
 integer,intent(out) :: indlmn(6,lmnmax)
 real(dp),intent(in) :: qgrid_ff(mqgrid_ff),qgrid_vl(mqgrid_vl)
 real(dp),intent(out) :: ffspl(mqgrid_ff,2,lnmax)
 real(dp),intent(out) :: vlspl(mqgrid_vl,2)

!Local variables ------------------------------
!scalars
 integer :: ib,icoremesh,il,ilm,ilmn,ilmn0,iln,imainmesh,imsh,iprojmesh,ipsploc
 integer :: ir,iread1,ishpfmesh,ivalemesh,ivlocmesh,j0lmn,jlm
 integer :: jlmn,jln,klmn,msz,nmesh,nval,pspversion,sz10,usexcnhat,vlocopt
 real(dp) :: fourpi,occ,rc,yp1,ypn
 logical :: has_dij0
 character(len=500) :: message
 type(pawrad_type) :: core_mesh,shpf_mesh,tproj_mesh,vale_mesh,vloc_mesh
!arrays
 integer,allocatable :: nprj(:),orbitals(:)
 real(dp),allocatable :: ncore(:)
 real(dp),allocatable :: shpf(:,:),tncore(:),tnvale(:),tproj(:,:),vhnzc(:),vlocr(:)
 real(dp),allocatable :: work1(:),work2(:),work3(:),work4(:)
 type(pawrad_type),allocatable :: radmesh(:)

!************************************************************************

!(27) Dij0 (for ij=1 to lmn_size*(lmn_size+1)/2)
!(28) comment(character)
!(29) Rhoij0 (for ij=1 to lmn_size*(lmn_size+1)/2)

!
!Comments:
!* mesh_type= type of radial mesh
!mesh_type=1 (regular grid): rad(i)=(i-1)*AA
!mesh_type=2 (logari. grid): rad(i)=AA*(exp[BB*(i-1)]-1)
!mesh_type=3 (logari. grid): rad(i>1)=AA*exp[BB*(i-2)] and rad(1)=0
!mesh_type=4 (logari. grid): rad(i)=-AA*ln[1-BB*(i-1)] with BB=1/n
!* radial shapefunction type
!shape_type=-1 ; gl(r)=numeric (read from psp file)
!shape_type= 1 ; gl(r)=k(r).r^l; k(r)=exp[-(r/sigma)**lambda]
!shape_type= 2 ; gl(r)=k(r).r^l; k(r)=[sin(pi*r/rshp)/(pi*r/rshp)]**2 if r<=rshp
!shape_type= 3 ; gl(r)=Alpha(1,l)*jl(q(1,l)*r)+Alpha(2,l)*jl(q(2,l)*r) for each l


!==========================================================

 DBG_ENTER("COLL")

!==========================================================

 fourpi=4*acos(-1.d0)
 pspversion=pspheads%pawheader%pawver
 ipsploc=ipsp2xml(ipsp)
 pawtab%basis_size=pspheads%pawheader%basis_size
 pawtab%lmn_size=pspheads%pawheader%lmn_size
 pawtab%l_size=pspheads%pawheader%l_size
 nmesh=paw_setup(ipsploc)%ngrid
 ABI_ALLOCATE(orbitals,(pawtab%basis_size))
 do ib=1,pawtab%basis_size
   orbitals(ib)=paw_setup(ipsploc)%valence_states%state(ib)%ll
 end do
 ABI_DATATYPE_ALLOCATE(radmesh,(nmesh))
 do imsh=1,nmesh
   radmesh(imsh)%mesh_type=-1
   radmesh(imsh)%rstep=zero
   radmesh(imsh)%lstep=zero
   select case(trim(paw_setup(ipsploc)%radial_grid(imsh)%eq))
     case("r=a*exp(d*i)")
       radmesh(imsh)%mesh_type=3
       radmesh(imsh)%mesh_size=paw_setup(ipsploc)%radial_grid(imsh)%iend-paw_setup(ipsploc)%radial_grid(imsh)%istart+2
       radmesh(imsh)%rstep=paw_setup(ipsploc)%radial_grid(imsh)%aa
       radmesh(imsh)%lstep=paw_setup(ipsploc)%radial_grid(imsh)%dd
     case("r=a*i/(1-b*i)")
       write(message, '(3a)' )&
&       '  the grid r=a*i/(1-b*i) is not implemented in ABINIT !',ch10,&
&       '  Action: check your psp file.'
       MSG_ERROR(message)
     case("r=a*i/(n-i)")
       radmesh(imsh)%mesh_type=5
       radmesh(imsh)%mesh_size=paw_setup(ipsploc)%radial_grid(imsh)%iend-paw_setup(ipsploc)%radial_grid(imsh)%istart+1
       radmesh(imsh)%rstep=paw_setup(ipsploc)%radial_grid(imsh)%aa
       radmesh(imsh)%lstep=dfloat(paw_setup(ipsploc)%radial_grid(imsh)%nn)
     case("r=a*(exp(d*i)-1)")
       radmesh(imsh)%mesh_type=2
       radmesh(imsh)%mesh_size=paw_setup(ipsploc)%radial_grid(imsh)%iend-paw_setup(ipsploc)%radial_grid(imsh)%istart+1
       if(paw_setup(ipsploc)%radial_grid(imsh)%istart==1)radmesh(imsh)%mesh_size=radmesh(imsh)%mesh_size+1
       radmesh(imsh)%rstep=paw_setup(ipsploc)%radial_grid(imsh)%aa
       radmesh(imsh)%lstep=paw_setup(ipsploc)%radial_grid(imsh)%dd
     case("r=d*i")
       radmesh(imsh)%mesh_type=1
       radmesh(imsh)%mesh_size=paw_setup(ipsploc)%radial_grid(imsh)%iend-paw_setup(ipsploc)%radial_grid(imsh)%istart+1
       if(paw_setup(ipsploc)%radial_grid(imsh)%istart==1)radmesh(imsh)%mesh_size=radmesh(imsh)%mesh_size+1
       radmesh(imsh)%rstep=paw_setup(ipsploc)%radial_grid(imsh)%dd
     case("r=(i/n+a)^5/a-a^4")
       write(message, '(3a)' )&
&       '  the grid r=(i/n+a)^5/a-a^4 is not implemented in ABINIT !',ch10,&
&       '  Action: check your psp file.'
       MSG_ERROR(message)
   end select
 end do

 pawtab%rpaw=pspheads%pawheader%rpaw

!Here reading shapefunction parameters
 pawtab%shape_type=pspheads%pawheader%shape_type
 pawtab%shape_lambda=-1;pawtab%shape_sigma=1.d99
 pawtab%rshp=pspheads%pawheader%rshp
 pawtab%shape_lambda=paw_setup(ipsploc)%shape_function%lamb
 if(trim(paw_setup(ipsploc)%shape_function%gtype)=="gauss")pawtab%shape_lambda=2
 pawtab%shape_sigma=paw_setup(ipsploc)%shape_function%rc
!If shapefunction type is gaussian, check exponent
 if (pawtab%shape_type==1) then
   if (pawtab%shape_lambda<2) then
     write(message, '(3a)' )&
&     '  For a gaussian shape function, exponent lambda must be >1 !',ch10,&
&     '  Action: check your psp file.'
     MSG_ERROR(message)
   end if
 end if

!If shapefunction type is Bessel, deduce here its parameters from rc
 if (pawtab%shape_type==3) then
   rc=pawtab%rshp;if (rc<1.d-8) rc=pawtab%rpaw
   do il=1,pawtab%l_size
     call shapebes(pawtab%shape_alpha(1:2,il),pawtab%shape_q(1:2,il),il-1,rc)
   end do
 end if

!==========================================================
!Mirror pseudopotential parameters to the output and log files

 write(message,'(a,i2)')' Pseudopotential format is: paw',pspversion
 call wrtout(ab_out,message,'COLL')
 call wrtout(std_out,  message,'COLL')
 write(message,'(2(a,i3),a,64i4)') &
& ' basis_size (lnmax)=',pawtab%basis_size,' (lmn_size=',&
& pawtab%lmn_size,'), orbitals=',orbitals(1:pawtab%basis_size)
 call wrtout(ab_out,message,'COLL')
 call wrtout(std_out,  message,'COLL')
 write(message,'(a,f11.8)')' Spheres core radius: rc_sph=',pawtab%rpaw
 call wrtout(ab_out,message,'COLL')
 call wrtout(std_out,  message,'COLL')
 write(message,'(a,i1,a)')' ',nmesh,' radial meshes are used:'
 call wrtout(ab_out,message,'COLL')
 call wrtout(std_out,  message,'COLL')
 do imsh=1,nmesh
   if (radmesh(imsh)%mesh_type==1) &
&   write(message,'(a,i1,a,i4,a,g12.5)') &
&   '  - mesh ',imsh,': r(i)=step*(i-1), size=',radmesh(imsh)%mesh_size,&
&   ' , step=',radmesh(imsh)%rstep
   if (radmesh(imsh)%mesh_type==2) &
&   write(message,'(a,i1,a,i4,2(a,g12.5))') &
&   '  - mesh ',imsh,': r(i)=AA*[exp(BB*(i-1))-1], size=',radmesh(imsh)%mesh_size,&
&   ' , AA=',radmesh(imsh)%rstep,' BB=',radmesh(imsh)%lstep
   if (radmesh(imsh)%mesh_type==3) &
&   write(message,'(a,i1,a,i4,2(a,g12.5))') &
&   '  - mesh ',imsh,': r(i)=AA*exp(BB*(i-2)), size=',radmesh(imsh)%mesh_size,&
&   ' , AA=',radmesh(imsh)%rstep,' BB=',radmesh(imsh)%lstep
   if (radmesh(imsh)%mesh_type==4) &
&   write(message,'(a,i1,a,i4,a,g12.5)') &
&   '  - mesh ',imsh,': r(i)=-AA*ln(1-(i-1)/n), n=size=',radmesh(imsh)%mesh_size,&
&   ' , AA=',radmesh(imsh)%rstep
   if (radmesh(imsh)%mesh_type==5) &
&   write(message,'(a,i1,a,i4,2(a,g12.5))') &
&   '  - mesh ',imsh,': r(i)=-AA*i/(NN-i)), n=size=',radmesh(imsh)%mesh_size,&
&   ' , AA=',radmesh(imsh)%rstep,' NN=',radmesh(imsh)%lstep
   call wrtout(ab_out,message,'COLL')
   call wrtout(std_out,  message,'COLL')
 end do
 if (pawtab%shape_type==-1) then
   write(message,'(a)')&
   ' Shapefunction is NUMERIC type: directly read from atomic data file'
   call wrtout(ab_out,message,'COLL')
   call wrtout(std_out,  message,'COLL')
 end if
 if (pawtab%shape_type==1) then
   write(message,'(2a,a,f6.3,a,i3)')&
&   ' Shapefunction is EXP type: shapef(r)=exp(-(r/sigma)**lambda)',ch10,&
&   '                            with sigma=',pawtab%shape_sigma,' and lambda=',pawtab%shape_lambda
   call wrtout(ab_out,message,'COLL')
   call wrtout(std_out,  message,'COLL')
 end if
 if (pawtab%shape_type==2) then
   write(message,'(a)')&
   ' Shapefunction is SIN type: shapef(r)=[sin(pi*r/rshp)/(pi*r/rshp)]**2'
   call wrtout(ab_out,message,'COLL')
   call wrtout(std_out,  message,'COLL')
 end if
 if (pawtab%shape_type==3) then
   write(message,'(a)')&
&   ' Shapefunction is BESSEL type: shapef(r,l)=aa(1,l)*jl(q(1,l)*r)+aa(2,l)*jl(q(2,l)*r)'
   call wrtout(ab_out,message,'COLL')
   call wrtout(std_out,  message,'COLL')
 end if
 if (pawtab%rshp<1.d-8) then
   write(message,'(a)') ' Radius for shape functions = sphere core radius'
 else
   write(message,'(a,f11.8)') ' Radius for shape functions = ',pawtab%rshp
 end if
 call wrtout(ab_out,message,'COLL')
 call wrtout(std_out,  message,'COLL')

!==========================================================
!Perfom tests

!Spin-orbit not yet implemented
 if (pspso>1) then
   write(message, '(a,a,a)' )&
&   '  Spin-orbit not yet implemented for PAW psps !',ch10,&
&   '  Action : check your pseudopotential or input file.'
   MSG_ERROR(message)
 end if

!Are lmax and orbitals compatibles ?
 if (lmax/=maxval(orbitals)) then
   write(message, '(a,a,a)' )&
&   '  lmax /= MAX(orbitals) !',ch10,&
&   '  Action: check your pseudopotential file.'
   MSG_ERROR(message)
 end if

!Are psps%lmnmax and pawtab%lmn_size compatibles ?
 if (lmnmax<pawtab%lmn_size) then
   message='  lmnmax < lmn size (read from pseudo) !'
   MSG_BUG(message)
 end if

!Only mesh_type=1,2, 3 or 5 allowed
 do imsh=1,nmesh
   if (radmesh(imsh)%mesh_type>5) then
     write(message, '(a,a,a)' )&
&     '  Only mesh types 1,2,3 or 5 allowed !',ch10,&
&     '  Action : check your pseudopotential or input file.'
     MSG_ERROR(message)
   end if
 end do

!==========================================================
!Initialize radial meshes

 do imsh=1,nmesh
   call pawrad_init(radmesh(imsh))
 end do

!==========================================================
!Read tabulated atomic data

!---------------------------------
!Read wave-functions (phi)

 do ib=1,pawtab%basis_size

   if (ib==1) then
     do imsh=1,nmesh
       if(trim(paw_setup(ipsploc)%ae_partial_wave(1)%grid)==trim(paw_setup(ipsploc)%radial_grid(imsh)%id)) then
         call pawrad_destroy(pawrad)
         call pawrad_init(pawrad,mesh_size=radmesh(imsh)%mesh_size,mesh_type=radmesh(imsh)%mesh_type, &
&         rstep=radmesh(imsh)%rstep,lstep=radmesh(imsh)%lstep,r_for_intg=pawtab%rpaw)
         mmax=radmesh(imsh)%mesh_size
         imainmesh=imsh
         cycle
       end if
     end do
   else if (trim(paw_setup(ipsploc)%ae_partial_wave(ib)%grid)/=trim(paw_setup(ipsploc)%radial_grid(imainmesh)%id)) then
     write(message, '(a,a,a)' )&
&     '  All Phi and tPhi must be given on the same radial mesh !',ch10,&
&     '  Action: check your pseudopotential file.'
     MSG_ERROR(message)
   end if
   pawtab%phi(1:pawrad%mesh_size,ib)=paw_setup(ipsploc)%ae_partial_wave(ib)%data(1:pawrad%mesh_size)*pawrad%rad(1:pawrad%mesh_size)
!  write(std_out,*)"PHI= ",ib
!  write(std_out,*)pawtab%phi(1:pawrad%mesh_size,ib)
 end do
 write(message,'(a,i4)') &
& ' mmax= ',mmax
 call wrtout(ab_out,message,'COLL')
 call wrtout(std_out,  message,'COLL')

!---------------------------------
!Read pseudo wave-functions (tphi)

 do ib=1,pawtab%basis_size

   if(trim(paw_setup(ipsploc)%pseudo_partial_wave(ib)%grid)/=trim(paw_setup(ipsploc)%radial_grid(imainmesh)%id)) then
     write(message, '(a,a,a)' )&
&     '  All Phi and tPhi must be given on the same radial mesh !',ch10,&
&     '  Action: check your pseudopotential file.'
     MSG_ERROR(message)
   end if
   pawtab%tphi(1:pawrad%mesh_size,ib)=&
&   paw_setup(ipsploc)%pseudo_partial_wave(ib)%data(1:pawrad%mesh_size)*pawrad%rad(1:pawrad%mesh_size)
!  write(std_out,*)"TPHI= ",ib
!  write(std_out,*)pawtab%tphi(1:pawrad%mesh_size,ib)
 end do
 write(message,'(a,i1)') &
& ' Radial grid used for partial waves is grid ',imainmesh
 call wrtout(ab_out,message,'COLL')
 call wrtout(std_out,  message,'COLL')

!---------------------------------
!Read projectors (tproj)
!Pending, add reading of Gaussian projectors:
!Nullify wavelet objects for safety:
 nullify(pawtab%wvl%pngau)
 nullify(pawtab%wvl%parg)
 nullify(pawtab%wvl%pfac)
 pawtab%wvl%ptotgau=0

 do ib=1,pawtab%basis_size
   if (ib==1) then
     do imsh=1,nmesh
       if(trim(paw_setup(ipsploc)%projector_function(1)%grid)==trim(paw_setup(ipsploc)%radial_grid(imsh)%id)) then
         iprojmesh=imsh
         cycle
       end if
     end do
     call pawrad_copy(radmesh(iprojmesh),tproj_mesh)
     ABI_ALLOCATE(tproj,(tproj_mesh%mesh_size,pawtab%basis_size))
   else if (trim(paw_setup(ipsploc)%projector_function(ib)%grid)/=trim(paw_setup(ipsploc)%radial_grid(iprojmesh)%id)) then
     write(message, '(a,a,a)' )&
&     '  All tprojectors must be given on the same radial mesh !',ch10,&
&     '  Action: check your pseudopotential file.'
     MSG_ERROR(message)
   end if
   tproj(1:tproj_mesh%mesh_size,ib)=paw_setup(ipsploc)%projector_function(ib)%data(1:tproj_mesh%mesh_size)&
&   *tproj_mesh%rad(1:tproj_mesh%mesh_size)
!  write(std_out,*)"TPROJ= ",ib
!  write(std_out,*)tproj(1:tproj_mesh%mesh_size,ib)
 end do
 write(message,'(a,i1)') &
& ' Radial grid used for projectors is grid ',iprojmesh
 call wrtout(ab_out,message,'COLL')
 call wrtout(std_out,  message,'COLL')

!---------------------------------
!Read core density (coredens)
 do imsh=1,nmesh
   if(trim(paw_setup(ipsploc)%ae_core_density%grid)==trim(paw_setup(ipsploc)%radial_grid(imsh)%id)) then
     icoremesh=imsh
     cycle
   end if
 end do
 call pawrad_copy(radmesh(icoremesh),core_mesh)
 if ((radmesh(icoremesh)%mesh_type/=pawrad%mesh_type).or.&
& (radmesh(icoremesh)%rstep    /=pawrad%rstep)    .or.&
& (radmesh(icoremesh)%lstep    /=pawrad%lstep)) then
   write(message, '(a,a,a,a,a)' )&
&   '  Ncore must be given on a radial mesh with the same',ch10,&
&   '  type and step(s) than the main radial mesh (mesh for Phi) !',ch10,&
&   '  Action: check your pseudopotential file.'
   MSG_ERROR(message)
 end if
 ABI_ALLOCATE(ncore,(core_mesh%mesh_size))
 ncore(1:core_mesh%mesh_size)=paw_setup(ipsploc)%ae_core_density%data(1:core_mesh%mesh_size)/sqrt(fourpi)
!write(std_out,*)"NCORE= "
!write(std_out,*)ncore(1:core_mesh%mesh_size)

!construct and save VH[z_NC] if requested
 if (pawtab%has_vhnzc == 1) then
   ABI_ALLOCATE(vhnzc,(core_mesh%mesh_size))
   call pawvhnzc(ncore,core_mesh,vhnzc,znucl)
   pawtab%VHnZC(1:pawrad%mesh_size)=vhnzc(1:pawrad%mesh_size)
   pawtab%has_vhnzc=2
   ABI_DEALLOCATE(vhnzc)
 end if
 pawtab%coredens(1:pawrad%mesh_size)=ncore(1:pawrad%mesh_size)

!---------------------------------
!Read pseudo core density (tcoredens)
 do imsh=1,nmesh
   if(trim(paw_setup(ipsploc)%pseudo_core_density%grid)==trim(paw_setup(ipsploc)%radial_grid(imsh)%id)) then
     iread1=imsh
     cycle
   end if
 end do
 if (iread1/=icoremesh) then
   write(message, '(a,a,a,a,a,a,a,a)' )&
&   '  Pseudized core density (tNcore) must be given',ch10,&
&   '  on the same radial mesh as core density (Ncore) !',ch10,&
&   '  Action: check your pseudopotential file.'
   MSG_ERROR(message)
 end if
 ABI_ALLOCATE(tncore,(core_mesh%mesh_size))
 tncore(1:core_mesh%mesh_size)=paw_setup(ipsploc)%pseudo_core_density%data(1:core_mesh%mesh_size)/sqrt(fourpi)
!write(std_out,*)"TNCORE= "
!write(std_out,*)tncore(1:core_mesh%mesh_size)
 if (maxval(abs(tncore(:)))<tol6) then
   pawtab%usetcore=0
   pawtab%tcoredens(1:pawrad%mesh_size,1)=zero
 else
   pawtab%usetcore=1
   pawtab%tcoredens(1:pawrad%mesh_size,1)=tncore(1:pawrad%mesh_size)
 end if
 write(message,'(a,i1)') &
& ' Radial grid used for (t)core density is grid ',icoremesh
 call wrtout(ab_out,message,'COLL')
 call wrtout(std_out,  message,'COLL')


!---------------------------------
!Read local pseudopotential=Vh(tn_zc) or Vbare

 if ((paw_setup(ipsploc)%blochl_local_ionic_potential%tread).and.&
& (pawtab%usexcnhat==-1.or.pawtab%usexcnhat==0.or.(pawtab%usexcnhat==1.and.&
& ((.not.paw_setup(ipsploc)%zero_potential%tread).or.(.not.paw_setup(ipsploc)%kresse_joubert_local_ionic_potential%tread))))) then
   usexcnhat=0;vlocopt=2
   do imsh=1,nmesh
     if(trim(paw_setup(ipsploc)%blochl_local_ionic_potential%grid)==trim(paw_setup(ipsploc)%radial_grid(imsh)%id)) then
       iread1=imsh
       cycle
     end if
   end do
   ivlocmesh=iread1
   call pawrad_copy(radmesh(ivlocmesh),vloc_mesh)
   ABI_ALLOCATE(vlocr,(vloc_mesh%mesh_size))
   vlocr(1:vloc_mesh%mesh_size)=paw_setup(ipsploc)%blochl_local_ionic_potential%data(1:vloc_mesh%mesh_size)/sqrt(fourpi)
!  write(std_out,*)"VBLOCHL= "
!  write(std_out,*)vlocr(1:vloc_mesh%mesh_size)
 else if((paw_setup(ipsploc)%zero_potential%tread).and.(pawtab%usexcnhat==-1.or.pawtab%usexcnhat==0.or.(pawtab%usexcnhat==1.and.&
&   (.not.paw_setup(ipsploc)%kresse_joubert_local_ionic_potential%tread)))) then
   usexcnhat=0;vlocopt=0
   do imsh=1,nmesh
     if(trim(paw_setup(ipsploc)%zero_potential%grid)==trim(paw_setup(ipsploc)%radial_grid(imsh)%id)) then
       iread1=imsh
       cycle
     end if
   end do
   ivlocmesh=iread1
   call pawrad_copy(radmesh(ivlocmesh),vloc_mesh)
   ABI_ALLOCATE(vlocr,(vloc_mesh%mesh_size))
   vlocr(1:vloc_mesh%mesh_size)=paw_setup(ipsploc)%zero_potential%data(1:vloc_mesh%mesh_size)/sqrt(fourpi)
!  write(std_out,*)"VBARE= "
!  write(std_out,*)vlocr(1:vloc_mesh%mesh_size)
 else if(paw_setup(ipsploc)%kresse_joubert_local_ionic_potential%tread) then
   usexcnhat=1;vlocopt=1
   do imsh=1,nmesh
     if(trim(paw_setup(ipsploc)%kresse_joubert_local_ionic_potential%grid)==trim(paw_setup(ipsploc)%radial_grid(imsh)%id)) then
       iread1=imsh
       cycle
     end if
   end do
   ivlocmesh=iread1
   call pawrad_copy(radmesh(ivlocmesh),vloc_mesh)
   ABI_ALLOCATE(vlocr,(vloc_mesh%mesh_size))
   vlocr(1:vloc_mesh%mesh_size)=paw_setup(ipsploc)%kresse_joubert_local_ionic_potential%data(1:vloc_mesh%mesh_size)/sqrt(fourpi)
!  write(std_out,*)"VKRESSE-JOUBERT= "
!  write(std_out,*)vlocr(1:vloc_mesh%mesh_size)
 else
   write(message, '(a,a,a,a,a)' )&
&   '  At least one local potential must be given',ch10,&
&   '  Action: check your pseudopotential file.'
   MSG_ERROR(message)
 end if

 write(message,'(a,i1)') &
& ' Radial grid used for Vloc is grid ',ivlocmesh
 call wrtout(ab_out,message,'COLL')
 call wrtout(std_out,  message,'COLL')

!---------------------------------
!Eventually read "numeric" shapefunctions (if shape_type=-1)
 if (pawtab%shape_type==-1) then
   do imsh=1,nmesh
     if(trim(paw_setup(ipsploc)%shape_function%grid)==trim(paw_setup(ipsploc)%radial_grid(imsh)%id)) then
       iread1=imsh
       cycle
     end if
   end do
   call pawrad_copy(radmesh(iread1),shpf_mesh)
   ishpfmesh=iread1
   ABI_ALLOCATE(shpf,(shpf_mesh%mesh_size,pawtab%l_size))
   shpf(1,1)=one
   do ir=2,shpf_mesh%mesh_size
     shpf(ir,1)=paw_setup(ipsploc)%shape_function%data(ir,1)
   end do
   sz10=size(paw_setup(ipsploc)%shape_function%data,2)
   if(sz10>=2) then
     do il=2,pawtab%l_size
       shpf(1,il)=zero
       do ir=2,shpf_mesh%mesh_size
         shpf(ir,il)=paw_setup(ipsploc)%shape_function%data(ir,il)
       end do
     end do
   else
     do il=2,pawtab%l_size
       shpf(1,il)=zero
       do ir=2,shpf_mesh%mesh_size
         shpf(ir,il)=paw_setup(ipsploc)%shape_function%data(ir,1)*shpf_mesh%rad(ir)**(il-1)
       end do
     end do
   end if
   write(message,'(a,i1)') &
&   ' Radial grid used for shape functions is grid ',iread1
   call wrtout(ab_out,message,'COLL')
   call wrtout(std_out,  message,'COLL')

!  Has to spline shape functions if mesh is not the "main" mesh
   if (ishpfmesh/=imainmesh) then
     msz=shpf_mesh%mesh_size
     ABI_ALLOCATE(work1,(msz))
     ABI_ALLOCATE(work2,(msz))
     ABI_ALLOCATE(work3,(msz))
     ABI_ALLOCATE(work4,(pawrad%mesh_size))
     work3(:)=shpf_mesh%rad(:)
     work4(:)=pawrad%rad(:)
     do il=1,pawtab%l_size
       call bound_deriv(shpf(1:msz,il),shpf_mesh,msz,yp1,ypn)
       call spline(work3,shpf(:,il),msz,yp1,ypn,work1)
       call splint(msz,work3,shpf(:,il),work1,pawrad%mesh_size,work4,pawtab%shapefunc(:,il))
     end do
     ABI_DEALLOCATE(work1)
     ABI_DEALLOCATE(work2)
     ABI_DEALLOCATE(work3)
     ABI_DEALLOCATE(work4)
   else
     pawtab%shapefunc(:,:)=shpf(:,:)
   end if
   ABI_DEALLOCATE(shpf)
 end if

!---------------------------------
!Read pseudo valence density 
 if (paw_setup(ipsploc)%pseudo_valence_density%tread) then
   do imsh=1,nmesh
     if(trim(paw_setup(ipsploc)%pseudo_valence_density%grid)==trim(paw_setup(ipsploc)%radial_grid(imsh)%id)) then
       iread1=imsh
       cycle
     end if
   end do
   ivalemesh=iread1
   call pawrad_copy(radmesh(iread1),vale_mesh)
   ABI_ALLOCATE(tnvale,(vale_mesh%mesh_size))
   tnvale(1:vale_mesh%mesh_size)=paw_setup(ipsploc)%pseudo_valence_density%data(1:vale_mesh%mesh_size)/sqrt(fourpi)
!  write(std_out,*)"TNVALE= "
!  write(std_out,*)tnvale(1:vale_mesh%mesh_size)
   pawtab%usetvale=1
   write(message,'(a,i1)') &
&   ' Radial grid used for pseudo valence density is grid ',ivalemesh
   call wrtout(ab_out,message,'COLL')
   call wrtout(std_out,  message,'COLL')
 else
   pawtab%usetvale=0
 end if

!---------------------------------
!Read initial guess of rhoij (rhoij0)

 pawtab%rhoij0=zero
 ilmn0=0
 do ib=1,pawtab%basis_size
   il=2*orbitals(ib)+1
   occ=paw_setup(ipsploc)%valence_states%state(ib)%ff
   if (occ<zero)occ=zero
   do ilmn=ilmn0+1,ilmn0+il
     pawtab%rhoij0(ilmn*(ilmn+1)/2)=occ/dble(il)
   end do
   ilmn0=ilmn0+il
 end do
!write(std_out,*)"RHOIJ0= "
!write(std_out,*)pawtab%rhoij0(1:pawtab%lmn2_size)
!==========================================================
!Initialize various parameters in structured data
!Some dims
 pawtab%l_size=2*maxval(orbitals)+1
 pawtab%mesh_size=pawrad%mesh_size
 pawtab%lmn2_size=pawtab%lmn_size*(pawtab%lmn_size+1)/2
 pawtab%core_mesh_size=pawrad%mesh_size
 pawtab%rcore=core_mesh%rad(pawtab%core_mesh_size)

!indlmn calculation (indices for (l,m,n) basis)
 ABI_ALLOCATE(nprj,(0:maxval(orbitals)))
 ilmn=0;iln=0;nprj=0
 do ib=1,pawtab%basis_size
   il=orbitals(ib)
   nprj(il)=nprj(il)+1
   iln=iln+1
   do ilm=1,2*il+1
     indlmn(1,ilmn+ilm)=il
     indlmn(2,ilmn+ilm)=ilm-(il+1)
     indlmn(3,ilmn+ilm)=nprj(il)
     indlmn(4,ilmn+ilm)=il*il+ilm
     indlmn(5,ilmn+ilm)=iln
     indlmn(6,ilmn+ilm)=1
   end do
   ilmn=ilmn+2*il+1
 end do
 ABI_DEALLOCATE(nprj)
 ABI_DEALLOCATE(orbitals)

!Are ilmn (found here) and pawtab%lmn_size compatibles ?
 if (ilmn/=pawtab%lmn_size) then
   write(message, '(a,a,a,a,a)' )&
&   '  Calculated lmn size differs from',ch10,&
&   '  lmn_size read from pseudo !',ch10,&
&   ' Action: check your pseudopotential file.'
   MSG_ERROR(message)
 end if
!---------------------------------
!Read Kij terms (kij0)
 pawtab%kij=zero
 nval=paw_setup(ipsploc)%valence_states%nval
 do jlmn=1,pawtab%lmn_size
   j0lmn=jlmn*(jlmn-1)/2
   jlm=indlmn(4,jlmn);jln=indlmn(5,jlmn)
   do ilmn=1,jlmn
     klmn=j0lmn+ilmn
     ilm=indlmn(4,ilmn);iln=indlmn(5,ilmn)
     if (ilm==jlm) pawtab%kij(klmn)=paw_setup(ipsploc)%kinetic_energy_differences%data(jln+(iln-1)*nval)
   end do
 end do
!write(std_out,*)"KIJ= "
!write(std_out,*)pawtab%kij(1:pawtab%lmn2_size)

 has_dij0=.false.

 call psp7calc(core_mesh,epsatm,ffspl,has_dij0,imainmesh,indlmn,ixc,lmnmax,lnmax,&
& mmax,mqgrid_ff,mqgrid_vl,ncore,nmesh,pawrad,pawtab,pawxcdev,pspversion,&
& qgrid_ff,qgrid_vl,&
& radmesh,tncore,tnvale,tproj,tproj_mesh,usexcnhat,vale_mesh,vloc_mesh,vlocopt,&
& vlocr,vlspl,xcccrc,xclevel,xc_denpos,zion,znucl)

!Nullify wvl+paw objects:
 nullify(pawtab%wvl%rholoc%rad)
 nullify(pawtab%wvl%rholoc%d)
 pawtab%wvl%rholoc%msz=0


!==========================================================
!Free temporary allocated space
 call pawrad_destroy_array(radmesh)
 call pawrad_destroy(tproj_mesh)
 call pawrad_destroy(core_mesh)
 call pawrad_destroy(vloc_mesh)
 ABI_DATATYPE_DEALLOCATE(radmesh)
 ABI_DEALLOCATE(vlocr)
 ABI_DEALLOCATE(ncore)
 ABI_DEALLOCATE(tncore)
 ABI_DEALLOCATE(tproj)
 if(pawtab%shape_type==-1) then
   call pawrad_destroy(shpf_mesh)
 end if
 if (paw_setup(ipsploc)%pseudo_valence_density%tread) then
   call pawrad_destroy(vale_mesh)
   ABI_DEALLOCATE(tnvale)
 end if

 DBG_EXIT('COLL')

end subroutine psp17in
!!***
