!{\src2tex{textfont=tt}}
!!****f* ABINIT/psp7in
!! NAME
!! psp7in
!!
!! FUNCTION
!! Initialize pspcod=7 ("PAW pseudopotentials"):
!! continue to read the corresponding file and compute the form factors
!!
!! COPYRIGHT
!! Copyright (C) 1998-2013 ABINIT group (GJ, FJ, MT, FB)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!  icoulomb==0 : usual reciprocal space computation
!!           =1 : free boundary conditions are used
!!  ipsp=id in the array of the currently read pseudo.
!!  ixc=exchange-correlation choice from main routine data file
!!  lloc=angular momentum choice of local pseudopotential
!!  lmax=value of lmax mentioned at the second line of the psp file
!!  psps <type(pseudopotential_type)>=at output, values depending on the read
!!                                    pseudo are set.
!!   | dimekb=dimension of ekb (see module defs_psp.f)
!!   | filpsp=name of formatted external file containing atomic psp data.
!!   | lmnmax=if useylm=1, max number of (l,m,n) comp. over all type of psps
!!   |           =if useylm=0, max number of (l,n)   comp. over all type of psps
!!   | lnmax=max. number of (l,n) components over all type of psps
!!   |           angular momentum of nonlocal pseudopotential
!!   | mpsang= 1+maximum angular momentum for nonlocal pseudopotentials
!!   | mpssoang= 1+maximum (spin*angular momentum) for nonlocal pseudopotentials
!!   | mqgrid_ff=dimension of q (or G) grid for nl form factors (array ffspl)
!!   | mqgrid_vl=dimension of q (or G) grid for Vloc (array vlspl)
!!   | n1xccc=dimension of xccc1d ; 0 if no XC core correction is used
!!   | optnlxccc=option for nl XC core correction
!!   | positron=0 if electron GS calculation
!!   |              1 if positron GS calculation
!!   |              2 if electron GS calculation in presence of the positron
!!   | pspso=spin-orbit characteristics, govern the content of ffspl and ekb
!!   |          if =0 : this input requires NO spin-orbit characteristics of the psp
!!   |          if =2 : this input requires HGH characteristics of the psp
!!   |          if =3 : this input requires HFN characteristics of the psp
!!   |          if =1 : this input will be changed at output to 1, 2, 3, according
!!   |                  to the intrinsic characteristics of the psp file
!!   | qgrid_ff(mqgrid_ff)=values of q on grid from 0 to qmax (bohr^-1) for nl form factors
!!   | qgrid_vl(mqgrid_vl)=values of q on grid from 0 to qmax (bohr^-1) for Vloc
!!   | usepaw= 0 for non paw calculation; =1 for paw calculation
!!   | usewvl=1 for wavelets calculation; 0 for non wavelets calculation
!!   | useylm=governs the way the nonlocal operator is to be applied:
!!   |            1=using Ylm, 0=using Legendre polynomials
!!   | vlspl_recipSpace=.true. if pseudo are expressed in reciprocal space.
!!   | znuclpsp=atomic number of atom as specified in input file to main routine
!!
!!  pawxcdev=choice of XC development (0=no dev. (use of angular mesh) ; 1 or 2=dev. on moments)
!!  xclevel= XC functional level
!!  zion=nominal valence of atom as specified in psp file
!!
!! OUTPUT
!!  epsatm=$ (4\pi)\int_0^\infty [r^2 (V(r)+\frac{Zv}{r}) dr]$(hartree)
!!  ffspl(psps%mqgrid_ff,2,psps%lnmax)=form factor f_l(q) and second derivative
!!   from spline fit for each angular momentum and each projector;
!!   if any, spin-orbit components begin at l=mpsang+1
!!  indlmn(6,psps%lmnmax)= array giving l,m,n,lm,ln,s for i=lmn
!!  paw_atdata <type(paw_atdata_type)>=data required to rewrite an atomic-data file
!!  pawrad <type(pawrad_type)>=paw radial mesh and related data
!!  pawtab <type(pawtab_type)>=paw tabulated starting data
!!  vlspl(mqgrid_vl,2)=q^2 Vloc(q) and second derivatives from spline fit
!!  xc_denpos= lowest allowed density (usually for the computation of the XC functionals)
!!  xcccrc=XC core correction cutoff radius (bohr) from psp file
!!
!! NOTES
!!  Spin-orbit not yet implemented (to be done)
!!
!! PARENTS
!!      pspatm_abinit
!!
!! CHILDREN
!!      pawrad_copy
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

#include "abi_common.h"

subroutine psp7in(epsatm,ffspl,icoulomb,indlmn,ipsp,ixc,lloc,lmax,mmax,&
&                  paw_atdata,pawrad,pawtab,pawxcdev,&
&                  psps,r2well,vlspl,xcccrc,&
&                  xclevel,xc_denpos,zion)

 use defs_basis
 use defs_datatypes
 use defs_abitypes
 use m_errors
 use m_splines
 use m_profiling

 use m_pawrad, only : pawrad_type, pawrad_init, pawrad_destroy, pawrad_destroy_array, &
&                     pawrad_copy, nderiv_gen, bound_deriv
 use m_pawtab, only : pawtab_type, wvlpaw_type

#if defined HAVE_DFT_LIBXC
  use libxc_functionals
#endif

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'psp7in'
 use interfaces_14_hidewrite
 use interfaces_64_atompaw
 use interfaces_65_psp, except_this_one => psp7in
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: icoulomb,ipsp,ixc,lloc,lmax,mmax,pawxcdev
 integer,intent(in) :: xclevel
 real(dp),intent(in) :: r2well,xc_denpos,zion
 real(dp),intent(out) :: epsatm,xcccrc
 type(paw_atdata_type),intent(out)::paw_atdata
 type(pawrad_type),intent(out) :: pawrad
 type(pawtab_type),intent(out) :: pawtab
 type(pseudopotential_type),intent(inout) :: psps
!arrays
 integer,intent(out) :: indlmn(6,psps%lmnmax)
 real(dp),intent(out) :: ffspl(psps%mqgrid_ff,2,psps%lnmax)
 real(dp),intent(out) :: vlspl(psps%mqgrid_vl,2)

!Local variables ------------------------------
!scalars
 integer :: creatorid,ib,icoremesh,ig
 integer :: ii,il,ilm,ilmn,iln,imainmesh,imsh,iprojmesh
 integer :: ir,iread1,iread2,ishpfmesh,ivalemesh,ivlocmesh,jj,jlmn
 integer :: msz,nmesh
 integer :: pngau_,pspversion,prjopt,ptotgau_,usexcnhat,vlocopt
 real(dp) :: rc,rread1,rread2,yp1,ypn
 complex(dp)::arg,fac
 logical :: has_dij0
 character :: blank=' ',numb=' '
 character(len=500) :: message,submessage
 character(len=80) :: pspline
 type(pawrad_type) :: core_mesh,shpf_mesh,tproj_mesh,vale_mesh,vloc_mesh
!arrays
 integer,allocatable :: nprj(:),orbitals(:)
 real(dp),allocatable :: ncore(:)
 real(dp),allocatable :: shpf(:,:),tncore(:),tnvale(:),tnvale0(:),tproj(:,:),vhnzc(:),vlocr(:)
 real(dp),allocatable :: work1(:),work2(:),work3(:),work4(:)
 complex(dp),allocatable:: ff(:)
 type(pawrad_type),allocatable :: radmesh(:)
! debug
! integer::ierr
! real(dp),allocatable::dfdr(:,:),raux1(:),raux2(:)
! end debug

!************************************************************************

!File format of formatted PAW psp input (the 3 first lines
!have already been read in calling -pspatm- routine) :

!(1) title (character) line
!(2) psps%znuclpsp(ipsp), zion, pspdat
!(3) pspcod, pspxc, lmax, lloc, mmax, r2well
!(4) psp_version, creatorID
!(5) basis_size, lmn_size
!(6) orbitals (for l=1 to basis_size)
!(7) number_of_meshes
!For imsh=1 to number_of_meshes
!(8)  mesh_index, mesh_type ,mesh_size, rad_step[, log_step]
!(9) r_cut(SPH)
!(10) shape_type, r_shape[, shapefunction arguments]
!For iln=1 to basis_size
!(11) comment(character)
!(12) radial mesh index for phi
!(13) phi(r) (for ir=1 to phi_meshsz)
!For iln=1 to basis_size
!(14) comment(character)
!(15) radial mesh index for tphi
!(16) tphi(r) (for ir=1 to phi_mesh_size)
!For iln=1 to basis_size
!(17) comment(character)
!(18) radial mesh index for tproj
!(19) tproj(r) (for ir=1 to proj_mesh_size)
!(20) comment(character)
!(21) radial mesh index for core_density
!(22) core_density (for ir=1 to core_mesh_size)
!(23) comment(character)
!(24) radial mesh index for pseudo_core_density
!(25) tcore_density (for ir=1 to core_mesh_size)
!(26) comment(character)
!(27) Dij0 (for ij=1 to lmn_size*(lmn_size+1)/2)
!(28) comment(character)
!(29) Rhoij0 (for ij=1 to lmn_size*(lmn_size+1)/2)
!(30) comment(character)
!(31) radial mesh index for Vloc, format of Vloc (0=Vbare, 1=VH(tnzc), 2=VH(tnzc) without nhat in XC)
!(32) Vloc(r) (for ir=1 to vloc_mesh_size)
!===== Following lines only if shape_type=-1 =====
!For il=1 to 2*max(orbitals)+1
!(33) comment(character)
!(34) radial mesh index for shapefunc
!(35) shapefunc(r)*gnorm(l)*r**l (for ir=1 to shape_mesh_size)
!(36) comment(character)
!(37) radial mesh index for pseudo_valence_density
!(38) tvale(r) (for ir=1 to vale_mesh_size)
!
!Comments:
!* psp_version= ID of PAW_psp version
!4 characters string of the form 'pawn' (with n varying)
!* creatorID= ID of psp generator
!creatorid=1xyz : psp generated from Holzwarth AtomPAW generator version x.yz
!creatorid=2xyz : psp generated from Vanderbilt ultra-soft generator version x.yz
!creatorid=-1: psp for tests (for developpers only)
!* mesh_type= type of radial mesh
!mesh_type=1 (regular grid): rad(i)=(i-1)*AA
!mesh_type=2 (logari. grid): rad(i)=AA*(exp[BB*(i-1)]-1)
!mesh_type=3 (logari. grid): rad(i>1)=AA*exp[BB*(i-2)] and rad(1)=0
!mesh_type=4 (logari. grid): rad(i)=-AA*ln[1-BB*(i-1)] with BB=1/n
!* radial shapefunction type
!shape_type=-1 ; gl(r)=numeric (read from psp file)
!shape_type= 1 ; gl(r)=k(r).r^l; k(r)=exp[-(r/sigma)**lambda]
!shape_type= 2 ; gl(r)=k(r).r^l; k(r)=[sin(pi*r/rshp)/(pi*r/rshp)]**2 if r<=rshp
!shape_type= 3 ; gl(r)=Alpha(1,l)*jl(q(1,l)*r)+Alpha(2,l)*jl(q(2,l)*r) for each l


!==========================================================

 DBG_ENTER("COLL")

!==========================================================
!Read psp version in line 4 of the header
 pspversion=1
 read (tmp_unit,'(a80)') pspline;pspline=adjustl(pspline)
 if (pspline(1:3)=="paw".or.pspline(1:3)=="PAW") &
& read(unit=pspline(4:80),fmt=*) pspversion
 if (pspversion<1.or.pspversion>5) then
   write(message, '(a,i2,a,a,a)' )&
&   '  This version of PAW psp file (',pspversion,') is not compatible with',ch10,&
&   '  current version of Abinit.'
   MSG_ERROR(message)
 end if

!==========================================================
!Read lines 4 to 11 of the header

!Have to maintain compatibility with Abinit v4.2.x
 if (pspversion==1) then
   read (unit=pspline,fmt=*) pawtab%basis_size,pawtab%lmn_size
   ABI_ALLOCATE(orbitals,(pawtab%basis_size))
   read(tmp_unit,*) (orbitals(ib), ib=1,pawtab%basis_size)
   pawtab%l_size=2*maxval(orbitals)+1
   nmesh=3
   ABI_DATATYPE_ALLOCATE(radmesh,(nmesh))
   read(tmp_unit,'(a80)') pspline
   radmesh(1)%lstep=zero
   read(unit=pspline,fmt=*,err=10,end=10) radmesh(1)%mesh_type,&
&   radmesh(1)%rstep,radmesh(1)%lstep
   10 read(tmp_unit,*) pawtab%rpaw
   read(tmp_unit,*) radmesh(1)%mesh_size,radmesh(2)%mesh_size,&
&   radmesh(3)%mesh_size
   read(tmp_unit,'(a80)') pspline
   pawtab%shape_lambda=-1;pawtab%shape_sigma=1.d99
   read(unit=pspline,fmt=*,err=11,end=11) pawtab%shape_type,&
&   pawtab%shape_lambda,pawtab%shape_sigma
   11 read(tmp_unit,*) creatorid
   if (pawtab%shape_type==3) pawtab%shape_type=-1
   radmesh(2)%mesh_type=radmesh(1)%mesh_type
   radmesh(3)%mesh_type=radmesh(1)%mesh_type
   radmesh(2)%rstep=radmesh(1)%rstep
   radmesh(3)%rstep=radmesh(1)%rstep
   radmesh(2)%lstep=radmesh(1)%lstep
   radmesh(3)%lstep=radmesh(1)%lstep
 else

!  Here psp file for Abinit 4.3+
   read (unit=pspline(5:80),fmt=*) creatorid
   read (tmp_unit,*) pawtab%basis_size,pawtab%lmn_size
   ABI_ALLOCATE(orbitals,(pawtab%basis_size))
   read(tmp_unit,*) (orbitals(ib), ib=1,pawtab%basis_size)
   pawtab%l_size=2*maxval(orbitals)+1
   read(tmp_unit,*) nmesh
   ABI_DATATYPE_ALLOCATE(radmesh,(nmesh))
   do imsh=1,nmesh
     rread2=zero
     read(tmp_unit,'(a80)') pspline
     read(unit=pspline,fmt=*,err=20,end=20) ii,iread1,iread2,rread1,rread2
     20 continue
     if (ii<=nmesh) then
       radmesh(ii)%mesh_type=iread1
       radmesh(ii)%mesh_size=iread2
       radmesh(ii)%rstep=rread1
       radmesh(ii)%lstep=rread2
     else
       write(message, '(3a)' )&
&       '  Index of mesh out of range !',ch10,&
&       '  Action : check your pseudopotential file.'
       MSG_ERROR(message)
     end if
   end do
   read(tmp_unit,*) pawtab%rpaw
   read(tmp_unit,'(a80)') pspline
   read(unit=pspline,fmt=*) pawtab%shape_type
   pawtab%shape_lambda=-1;pawtab%shape_sigma=1.d99
 end if

!Here reading shapefunction parameters for Abinit 4.3...4.5
 if (pspversion==2) then
   if (pawtab%shape_type==1) read(unit=pspline,fmt=*) ii,pawtab%shape_lambda,pawtab%shape_sigma
   if (pawtab%shape_type==3) pawtab%shape_type=-1
   pawtab%rshp=zero

!  Here reading shapefunction parameters for Abinit 4.6+
 else if (pspversion>=3) then
   pawtab%rshp=zero
   if (pawtab%shape_type==-1) read(unit=pspline,fmt=*,err=21,end=21) ii,pawtab%rshp
   if (pawtab%shape_type== 1) read(unit=pspline,fmt=*,err=21,end=21) ii,pawtab%rshp, &
&   pawtab%shape_lambda,pawtab%shape_sigma
   if (pawtab%shape_type== 2) read(unit=pspline,fmt=*,err=21,end=21) ii,pawtab%rshp
   if (pawtab%shape_type== 3) read(unit=pspline,fmt=*,err=21,end=21) ii,pawtab%rshp
 end if
 21 continue
!If shapefunction type is gaussian, check exponent
 if (pawtab%shape_type==1) then
   if (pawtab%shape_lambda<2) then
     write(message, '(3a)' )&
&     '  For a gaussian shape function, exponent lambda must be >1 !',ch10,&
&     '  Action: check your psp file.'
     MSG_ERROR(message)
   end if
 end if
!If shapefunction type is Bessel, deduce here its parameters from rc
 if (pawtab%shape_type==3) then
   rc=pawtab%rshp;if (rc<1.d-8) rc=pawtab%rpaw
   do il=1,pawtab%l_size
     call shapebes(pawtab%shape_alpha(1:2,il),pawtab%shape_q(1:2,il),il-1,rc)
   end do
 end if

!==========================================================
!Mirror pseudopotential parameters to the output and log files

 write(message,'(a,i1)')' Pseudopotential format is: paw',pspversion
 call wrtout(ab_out,message,'COLL')
 call wrtout(std_out,  message,'COLL')
 write(message,'(2(a,i3),a,64i4)') &
& ' basis_size (lnmax)=',pawtab%basis_size,' (lmn_size=',&
& pawtab%lmn_size,'), orbitals=',orbitals(1:pawtab%basis_size)
 call wrtout(ab_out,message,'COLL')
 call wrtout(std_out,  message,'COLL')
 write(message,'(a,f11.8)')' Spheres core radius: rc_sph=',pawtab%rpaw
 call wrtout(ab_out,message,'COLL')
 call wrtout(std_out,  message,'COLL')
 write(message,'(a,i1,a)')' ',nmesh,' radial meshes are used:'
 call wrtout(ab_out,message,'COLL')
 call wrtout(std_out,  message,'COLL')
 do imsh=1,nmesh
   if (radmesh(imsh)%mesh_type==1) &
&   write(message,'(a,i1,a,i4,a,g12.5)') &
&   '  - mesh ',imsh,': r(i)=step*(i-1), size=',radmesh(imsh)%mesh_size,&
&   ' , step=',radmesh(imsh)%rstep
   if (radmesh(imsh)%mesh_type==2) &
&   write(message,'(a,i1,a,i4,2(a,g12.5))') &
&   '  - mesh ',imsh,': r(i)=AA*[exp(BB*(i-1))-1], size=',radmesh(imsh)%mesh_size,&
&   ' , AA=',radmesh(imsh)%rstep,' BB=',radmesh(imsh)%lstep
   if (radmesh(imsh)%mesh_type==3) &
&   write(message,'(a,i1,a,i4,2(a,g12.5))') &
&   '  - mesh ',imsh,': r(i)=AA*exp(BB*(i-2)), size=',radmesh(imsh)%mesh_size,&
&   ' , AA=',radmesh(imsh)%rstep,' BB=',radmesh(imsh)%lstep
   if (radmesh(imsh)%mesh_type==4) &
&   write(message,'(a,i1,a,i4,a,g12.5)') &
&   '  - mesh ',imsh,': r(i)=-AA*ln(1-(i-1)/n), n=size=',radmesh(imsh)%mesh_size,&
&   ' , AA=',radmesh(imsh)%rstep
   call wrtout(ab_out,message,'COLL')
   call wrtout(std_out,  message,'COLL')
 end do
 if (pawtab%shape_type==-1) then
   write(message,'(a)')&
   ' Shapefunction is NUMERIC type: directly read from atomic data file'
   call wrtout(ab_out,message,'COLL')
   call wrtout(std_out,  message,'COLL')
 end if
 if (pawtab%shape_type==1) then
   write(message,'(2a,a,f6.3,a,i3)')&
&   ' Shapefunction is EXP type: shapef(r)=exp(-(r/sigma)**lambda)',ch10,&
&   '                            with sigma=',pawtab%shape_sigma,' and lambda=',pawtab%shape_lambda
   call wrtout(ab_out,message,'COLL')
   call wrtout(std_out,  message,'COLL')
 end if
 if (pawtab%shape_type==2) then
   write(message,'(a)')&
   ' Shapefunction is SIN type: shapef(r)=[sin(pi*r/rshp)/(pi*r/rshp)]**2'
   call wrtout(ab_out,message,'COLL')
   call wrtout(std_out,  message,'COLL')
 end if
 if (pawtab%shape_type==3) then
   write(message,'(a)')&
&   ' Shapefunction is BESSEL type: shapef(r,l)=aa(1,l)*jl(q(1,l)*r)+aa(2,l)*jl(q(2,l)*r)'
   call wrtout(ab_out,message,'COLL')
   call wrtout(std_out,  message,'COLL')
 end if
 if (pawtab%rshp<1.d-8) then
   write(message,'(a)') ' Radius for shape functions = sphere core radius'
 else
   write(message,'(a,f11.8)') ' Radius for shape functions = ',pawtab%rshp
 end if
 call wrtout(ab_out,message,'COLL')
 call wrtout(std_out,  message,'COLL')

!==========================================================
!Perfom tests

!Spin-orbit not yet implemented
 if (any(psps%pspso(:)>1)) then
   write(message, '(a,a,a)' )&
&   '  Spin-orbit not yet implemented for PAW psps !',ch10,&
&   '  Action : check your pseudopotential or input file.'
   MSG_ERROR(message)
 end if

!Are lmax and orbitals compatibles ?
 if (lmax/=maxval(orbitals)) then
   write(message, '(a,a,a)' )&
&   '  lmax /= MAX(orbitals) !',ch10,&
&   '  Action: check your pseudopotential file.'
   MSG_ERROR(message)
 end if

!Are psps%lmnmax and pawtab%lmn_size compatibles ?
 if (psps%lmnmax<pawtab%lmn_size) then
   message='  psps%lmnmax < lmn size (read from pseudo) !'
   MSG_BUG(message)
 end if

!Only mesh_type=1,2, 3 or 4 allowed
 do imsh=1,nmesh
   if (radmesh(imsh)%mesh_type>4) then
     write(message, '(a,a,a)' )&
&     '  Only mesh types 1,2, 3 or 4 allowed !',ch10,&
&     '  Action : check your pseudopotential or input file.'
     MSG_ERROR(message)
   end if
 end do

!==========================================================
!Initialize radial meshes

 do imsh=1,nmesh
   call pawrad_init(radmesh(imsh))
 end do

!==========================================================
!Read tabulated atomic data

!---------------------------------
!Read wave-functions (phi)
 do ib=1,pawtab%basis_size
   read (tmp_unit,*)
   if (pspversion==1) iread1=1
   if (pspversion>1) read (tmp_unit,*) iread1
   if (ib==1) then
     call pawrad_destroy(pawrad)
     call pawrad_init(pawrad,mesh_size=radmesh(iread1)%mesh_size,mesh_type=radmesh(iread1)%mesh_type,&
&     rstep=radmesh(iread1)%rstep,lstep=radmesh(iread1)%lstep,r_for_intg=pawtab%rpaw)
     imainmesh=iread1
   else if (iread1/=imainmesh) then
     write(message, '(a,a,a)' )&
&     '  All Phi and tPhi must be given on the same radial mesh !',ch10,&
&     '  Action: check your pseudopotential file.'
     MSG_ERROR(message)
   end if
   read (tmp_unit,*) (pawtab%phi(ir,ib),ir=1,pawrad%mesh_size)
 end do

!---------------------------------
!Read pseudo wave-functions (tphi)
 do ib=1,pawtab%basis_size
   read (tmp_unit,*)
   if (pspversion==1) iread1=1
   if (pspversion>1) read (tmp_unit,*) iread1
   if (iread1/=imainmesh) then
     write(message, '(a,a,a)' )&
&     '  All Phi and tPhi must be given on the same radial mesh !',ch10,&
&     '  Action: check your pseudopotential file.'
     MSG_ERROR(message)
   end if
   read (tmp_unit,*) (pawtab%tphi(ir,ib),ir=1,pawrad%mesh_size)
 end do
 write(message,'(a,i1)') &
& ' Radial grid used for partial waves is grid ',imainmesh
 call wrtout(ab_out,message,'COLL')
 call wrtout(std_out,  message,'COLL')

!---------------------------------
!Read projectors (tproj)
 jj=0
 do ib=1,pawtab%basis_size
   read (tmp_unit,*)
   if (pspversion==1) iread1=2
   if (pspversion>1) read (tmp_unit,*) iread1
   if (ib==1) then
     iprojmesh=iread1
     if(iread1==-1) iread1=2 !for wvl+paw
     call pawrad_copy(radmesh(iread1),tproj_mesh)
     ABI_ALLOCATE(tproj,(tproj_mesh%mesh_size,pawtab%basis_size))
   else if (iread1/=iprojmesh) then
     write(message, '(a,a,a)' )&
&     '  All tprojectors must be given on the same radial mesh !',ch10,&
&     '  Action: check your pseudopotential file.'
     MSG_ERROR(message)
   end if
!  
   if(iprojmesh /= -1) then 
!    read projectors from a mesh
     read (tmp_unit,*) (tproj(ir,ib),ir=1,tproj_mesh%mesh_size)
!    Nullify wavelet objects for safety:
     nullify(pawtab%wvl%pngau)
     nullify(pawtab%wvl%parg)
     nullify(pawtab%wvl%pfac)
     pawtab%wvl%ptotgau=0
   else
!    read Gaussian coefficients
     read(tmp_unit,*) pngau_, ptotgau_ !total number of gaussians
     if(ib==1) then
       pawtab%wvl%ptotgau=ptotgau_
       if(associated(pawtab%wvl%parg)) then 
         ABI_DEALLOCATE(pawtab%wvl%parg)   
       end if
       if(associated(pawtab%wvl%pfac)) then
         ABI_DEALLOCATE(pawtab%wvl%pfac)
       end if
       if(associated(pawtab%wvl%pngau)) then
         ABI_DEALLOCATE(pawtab%wvl%pngau)  
       end if
       ABI_ALLOCATE(pawtab%wvl%pngau,(pawtab%basis_size))
       ABI_ALLOCATE(pawtab%wvl%parg,(2,pawtab%wvl%ptotgau))
       ABI_ALLOCATE(pawtab%wvl%pfac,(2,pawtab%wvl%ptotgau))
     else
       if(pawtab%wvl%ptotgau .ne. ptotgau_) then
         write(message, '(a,a,a,a,a,a)' ) ch10,&
&         ' psp7in: ERROR -',ch10,&
&         '  total number of gaussians, should be the same for all projectors !',ch10,&
&         '  Action: check your pseudopotential file.'
         call wrtout(ab_out,message,'COLL')
         MSG_ERROR(message)
       end if
     end if !ib==1
     read(tmp_unit,*)(pawtab%wvl%parg(:,ii),ii=jj+1,jj+pngau_)
     read(tmp_unit,*)(pawtab%wvl%pfac(:,ii),ii=jj+1,jj+pngau_)
     pawtab%wvl%pngau(ib)=pngau_
     jj=jj+pngau_
   end if !iprojmesh==-1
 end do
 write(message,'(a,i2)') &
& ' Radial grid used for projectors is grid ',iprojmesh
 call wrtout(ab_out,message,'COLL')
 call wrtout(std_out,  message,'COLL')
!
!For wvl+paw:
!1) If projectors are given as sums of gaussians, then
!calculate projectors in the grid. (not used)
 if(psps%usewvl==1 ) then
   if(iprojmesh == -1) then
!    1) calculate projectors in the grid
     ABI_ALLOCATE(ff,(tproj_mesh%mesh_size))
     ig=0
     do ib=1,pawtab%basis_size
       ff(:)=czero
       do jj=1,pawtab%wvl%pngau(ib)
         ig=ig+1
         arg=cmplx(pawtab%wvl%parg(1,ig),pawtab%wvl%parg(2,ig))
         fac=cmplx(pawtab%wvl%pfac(1,ig),pawtab%wvl%pfac(2,ig))
         ff(:)=ff(:)+fac*exp(arg*tproj_mesh%rad(:)**2)
       end do !igau
       tproj(:,ib)=real(ff(:))
!      debug:
!      ii=500+ib
!      do jj=1,tproj_mesh%mesh_size
!      write(ii,'(2(f20.15,x))') tproj_mesh%rad(jj),tproj(jj,ib)
!      end do
!      end debug
     end do !ib
     ABI_DEALLOCATE(ff)
   end if !iprojmesh .ne. -1
 end if !usewvl==1

!---------------------------------
!Read core density (coredens)
 read (tmp_unit,*)
 if (pspversion==1) iread1=1
 if (pspversion>1) read (tmp_unit,*) iread1
 icoremesh=iread1
 call pawrad_copy(radmesh(iread1),core_mesh)
 if ((radmesh(icoremesh)%mesh_type/=pawrad%mesh_type).or.&
& (radmesh(icoremesh)%rstep    /=pawrad%rstep)    .or.&
& (radmesh(icoremesh)%lstep    /=pawrad%lstep)) then
   write(message, '(a,a,a,a,a)' )&
&   '  Ncore must be given on a radial mesh with the same',ch10,&
&   '  type and step(s) than the main radial mesh (mesh for Phi) !',ch10,&
&   '  Action: check your pseudopotential file.'
   MSG_ERROR(message)
 end if
 ABI_ALLOCATE(ncore,(core_mesh%mesh_size))
 read (tmp_unit,*) (ncore(ir),ir=1,core_mesh%mesh_size)
!construct and save VH[z_NC] if requested
 if (pawtab%has_vhnzc == 1) then
   ABI_ALLOCATE(vhnzc,(core_mesh%mesh_size))
   call pawvhnzc(ncore,core_mesh,vhnzc,psps%znuclpsp(ipsp))
   pawtab%VHnZC(1:pawrad%mesh_size)=vhnzc(1:pawrad%mesh_size)
   pawtab%has_vhnzc=2
   ABI_DEALLOCATE(vhnzc)
 end if
!In wvl+paw save core_mesh_size
 if(psps%usewvl==1 .or. icoulomb .ne. 0) then
   pawtab%core_mesh_size=core_mesh%mesh_size
   if(associated(pawtab%coredens)) then
     ABI_DEALLOCATE(pawtab%coredens)
   end if
   ABI_ALLOCATE(pawtab%coredens ,(pawtab%core_mesh_size))
   if(associated(pawtab%tcoredens)) then
     ABI_DEALLOCATE(pawtab%tcoredens)
   end if
   ABI_ALLOCATE(pawtab%tcoredens,(pawtab%core_mesh_size,6))
 else
   pawtab%core_mesh_size=pawrad%mesh_size
 end if
 pawtab%rcore=core_mesh%rad(pawtab%core_mesh_size)
 pawtab%coredens(1:pawtab%core_mesh_size)=ncore(1:pawtab%core_mesh_size)

!---------------------------------
!Read pseudo core density (tcoredens)
 read (tmp_unit,*)
 if (pspversion==1) iread1=1
 if (pspversion>1) read (tmp_unit,*) iread1
 if (iread1/=icoremesh) then
   write(message, '(a,a,a,a,a,a,a,a)' )&
&   '  Pseudized core density (tNcore) must be given',ch10,&
&   '  on the same radial mesh as core density (Ncore) !',ch10,&
&   '  Action: check your pseudopotential file.'
   MSG_ERROR(message)
 end if
 ABI_ALLOCATE(tncore,(core_mesh%mesh_size))
 read (tmp_unit,*) (tncore(ir),ir=1,core_mesh%mesh_size)
 if (maxval(abs(tncore(:)))<tol6) then
   pawtab%usetcore=0
   pawtab%tcoredens(1:pawtab%core_mesh_size,:)=zero
 else
   pawtab%usetcore=1
   pawtab%tcoredens(1:pawtab%core_mesh_size,1)=tncore(1:pawtab%core_mesh_size)
 end if
 write(message,'(a,i1)') &
& ' Radial grid used for (t)core density is grid ',icoremesh
 call wrtout(ab_out,message,'COLL')
 call wrtout(std_out,  message,'COLL')

!
!For PAW+WVL calculate up to the 5th derivative
!of tcoredens
 if(psps%usewvl==1 .or. icoulomb .ne. 0) then
   call psp7cc_wvl(core_mesh,pawtab%tcoredens)
 end if

!---------------------------------
!Read frozen part of Dij terms (dij0)
 read (tmp_unit,*)
 read (tmp_unit,*) (pawtab%dij0(ib),ib=1,pawtab%lmn2_size)

!---------------------------------
!Read initial guess of rhoij (rhoij0)
 read (tmp_unit,*)
 read (tmp_unit,*) (pawtab%rhoij0(ib),ib=1,pawtab%lmn2_size)

!---------------------------------
!Read local pseudopotential=Vh(tn_zc) or Vbare
 read (tmp_unit,*)
 if (pspversion==1) ivlocmesh=3
 vlocopt=1
 if (pspversion==2) then
   read (tmp_unit,*) ivlocmesh
 else if (pspversion>2) then
!  read (tmp_unit,fmt=*,err=30,end=30) ivlocmesh,vlocopt
   message=blank
   read (tmp_unit,fmt='(a)') message
   read (message,fmt=*) ivlocmesh
   write(numb,'(i1)')ivlocmesh
   ii=index(message,numb)
   if(len_trim(trim(message(ii+1:)))/=0)then
     submessage=trim(message(ii+1:))
     if(len_trim(submessage)/=0)then
       do ii=1,len_trim(submessage)
         numb=submessage(ii:ii)
         if(numb==blank)cycle
         jj=index('0123456789',numb)
         if(jj<1 .or. jj>10)exit
         vlocopt=jj-1
       end do
     end if
   end if
 end if
 usexcnhat=0;if (vlocopt==1) usexcnhat=1
 call pawrad_copy(radmesh(ivlocmesh),vloc_mesh)
 ABI_ALLOCATE(vlocr,(vloc_mesh%mesh_size))
 read (tmp_unit,*) (vlocr(ir),ir=1,vloc_mesh%mesh_size)
 write(message,'(a,i1)') &
& ' Radial grid used for Vloc is grid ',ivlocmesh
 call wrtout(ab_out,message,'COLL')
 call wrtout(std_out,  message,'COLL')

!---------------------------------
!Eventually read "numeric" shapefunctions (if shape_type=-1)
 if (pawtab%shape_type==-1) then
   do il=1,pawtab%l_size
     read (tmp_unit,*)
     if (pspversion==1) iread1=1
     if (pspversion>1) read (tmp_unit,*) iread1
     if (il==1) then
       call pawrad_copy(radmesh(iread1),shpf_mesh)
       ishpfmesh=iread1
       ABI_ALLOCATE(shpf,(shpf_mesh%mesh_size,pawtab%l_size))
     else if (iread1/=ishpfmesh) then
       write(message, '(a,a,a)' )&
&       '  All shape functions must be given on the same radial mesh !',ch10,&
&       '  Action: check your pseudopotential file.'
       MSG_ERROR(message)
     end if
     read (tmp_unit,*) (shpf(ir,il),ir=1,shpf_mesh%mesh_size)
   end do
   write(message,'(a,i1)') &
&   ' Radial grid used for shape functions is grid ',iread1
   call wrtout(ab_out,message,'COLL')
   call wrtout(std_out,  message,'COLL')

!  Has to spline shape functions if mesh is not the "main" mesh
   if (ishpfmesh/=imainmesh) then
     msz=shpf_mesh%mesh_size
     ABI_ALLOCATE(work1,(msz))
     ABI_ALLOCATE(work2,(msz))
     ABI_ALLOCATE(work3,(msz))
     ABI_ALLOCATE(work4,(pawrad%mesh_size))
     work3(:)=shpf_mesh%rad(:)
     work4(:)=pawrad%rad(:)
     do il=1,pawtab%l_size
       call bound_deriv(shpf(1:msz,il),shpf_mesh,msz,yp1,ypn)
       call spline(work3,shpf(:,il),msz,yp1,ypn,work1)
       call splint(msz,work3,shpf(:,il),work1,pawrad%mesh_size,work4,pawtab%shapefunc(:,il))
     end do
     ABI_DEALLOCATE(work1)
     ABI_DEALLOCATE(work2)
     ABI_DEALLOCATE(work3)
     ABI_DEALLOCATE(work4)
   else
     pawtab%shapefunc(:,:)=shpf(:,:)
   end if
   ABI_DEALLOCATE(shpf)
 end if

!---------------------------------
!Read pseudo valence density (if psp version >=4)
 if (pspversion>=4) then
   read (tmp_unit,*)
   read (tmp_unit,*) iread1
   ivalemesh=iread1
   call pawrad_copy(radmesh(iread1),vale_mesh)
   ABI_ALLOCATE(tnvale,(vale_mesh%mesh_size))
   read (tmp_unit,*) (tnvale(ir),ir=1,vale_mesh%mesh_size)
   pawtab%usetvale=1
   write(message,'(a,i1)') &
&   ' Radial grid used for pseudo valence density is grid ',ivalemesh
   call wrtout(ab_out,message,'COLL')
   call wrtout(std_out,  message,'COLL')
!  save tnvale, if we will rewrite the atomic data file
   if(psps%usewvl==1 .and. iprojmesh /= -1) then 
     ABI_ALLOCATE(tnvale0,(vale_mesh%mesh_size))
     tnvale0=tnvale
   end if
 else
   pawtab%usetvale=0
 end if

!==========================================================
!Initialize various parameters in structured data

!Some dims
 pawtab%l_size=2*maxval(orbitals)+1
 pawtab%mesh_size=pawrad%mesh_size
 pawtab%lmn2_size=pawtab%lmn_size*(pawtab%lmn_size+1)/2

!indlmn calculation (indices for (l,m,n) basis)
 ABI_ALLOCATE(nprj,(0:maxval(orbitals)))
 ilmn=0;iln=0;nprj=0
 do ib=1,pawtab%basis_size
   il=orbitals(ib)
   nprj(il)=nprj(il)+1
   iln=iln+1
   do ilm=1,2*il+1
     indlmn(1,ilmn+ilm)=il
     indlmn(2,ilmn+ilm)=ilm-(il+1)
     indlmn(3,ilmn+ilm)=nprj(il)
     indlmn(4,ilmn+ilm)=il*il+ilm
     indlmn(5,ilmn+ilm)=iln
     indlmn(6,ilmn+ilm)=1
   end do
   ilmn=ilmn+2*il+1
 end do
 ABI_DEALLOCATE(nprj)

!Are ilmn (found here) and pawtab%lmn_size compatibles ?
 if (ilmn/=pawtab%lmn_size) then
   write(message, '(a,a,a,a,a)' )&
&   '  Calculated lmn size differs from',ch10,&
&   '  lmn_size read from pseudo !',ch10,&
&   ' Action: check your pseudopotential file.'
   MSG_ERROR(message)
 end if

 has_dij0=.true.
 call psp7calc(core_mesh,epsatm,ffspl,has_dij0,imainmesh,indlmn,ixc,psps%lmnmax,psps%lnmax,&
& mmax,psps%mqgrid_ff,psps%mqgrid_vl,ncore,nmesh,pawrad,pawtab,pawxcdev,pspversion,&
& psps%qgrid_ff,psps%qgrid_vl,&
& radmesh,tncore,tnvale,tproj,tproj_mesh,usexcnhat,vale_mesh,vloc_mesh,vlocopt,&
& vlocr,vlspl,xcccrc,xclevel,xc_denpos,zion,psps%znuclpsp(ipsp))

 if(psps%usewvl==1 .or. icoulomb > 0) then
   call psp7wvl1(pawtab,tnvale,psps%usewvl,vale_mesh,vloc_mesh,vlocr)
 else
   nullify(pawtab%wvl%rholoc%rad)
   nullify(pawtab%wvl%rholoc%d)
   pawtab%wvl%rholoc%msz=0
 end if

!==================================================
!WVL+PAW: Fill paw_atdata object
 if(iprojmesh .ne. -1 .and. psps%usewvl==1) then
   call fill_paw_atdata()
 end if

!==========================================================
!Free temporary allocated space
 call pawrad_destroy_array(radmesh)
 call pawrad_destroy(tproj_mesh)
 call pawrad_destroy(core_mesh)
 call pawrad_destroy(vloc_mesh)
 ABI_DATATYPE_DEALLOCATE(radmesh)
 ABI_DEALLOCATE(orbitals)
 ABI_DEALLOCATE(vlocr)
 ABI_DEALLOCATE(ncore)
 ABI_DEALLOCATE(tncore)
 ABI_DEALLOCATE(tproj)
 if(pawtab%shape_type==-1) then
   call pawrad_destroy(shpf_mesh)
 end if
 if (pspversion>=4)  then
   call pawrad_destroy(vale_mesh)
   ABI_DEALLOCATE(tnvale)
   if (.FALSE.) call wrtout(std_out,"Another silly trick for XLF","COLL")
 end if

 DBG_EXIT('COLL')

 CONTAINS
!!***

!!****f* psp7in/fill_paw_atdata
!! NAME
!!  fill_paw_atdata
!!
!! FUNCTION
!!  Fill paw_atdata object with all infos needed to
!!  write again an atomic-data file
!!
!! COPYRIGHT
!!  Copyright (C) 2013 ABINIT group (T. Rangel)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!
!! OUTPUT
!!
!! SIDE EFFECTS
!!
!! NOTES
!!
!! PARENTS
!!      psp7in
!!
!! CHILDREN
!!      pawrad_copy
!!
!! SOURCE

 subroutine fill_paw_atdata()


!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'fill_paw_atdata'
!End of the abilint section

  implicit none
  !
!Arguments ------------------------------------

!Local variables ------------------------------
!************************************************************************
   integer :: imesh

!  Allocate arrays
   ABI_ALLOCATE(paw_atdata%orbitals,(pawtab%basis_size))
   ABI_DATATYPE_ALLOCATE(paw_atdata%radmesh,(nmesh))
   ABI_ALLOCATE(paw_atdata%ncore,(core_mesh%mesh_size))
   ABI_ALLOCATE(paw_atdata%tncore,(core_mesh%mesh_size))
   ABI_ALLOCATE(paw_atdata%tnvale0,(vale_mesh%mesh_size))
   ABI_ALLOCATE(paw_atdata%tproj,(tproj_mesh%mesh_size,pawtab%basis_size))
   ABI_ALLOCATE(paw_atdata%vlocr,(vloc_mesh%mesh_size))

!  write(paw_atdata%pspfmt,'("paw",i1)')pspversion
!  integers:
   paw_atdata%nmesh=nmesh
   paw_atdata%pspversion=pspversion
   paw_atdata%imainmesh=imainmesh
   paw_atdata%icoremesh=icoremesh
   paw_atdata%iprojmesh=iprojmesh
   paw_atdata%ishpfmesh=ishpfmesh
   paw_atdata%ivalemesh=ivalemesh
   paw_atdata%ivlocmesh=ivlocmesh
   paw_atdata%vlocopt=vlocopt
   paw_atdata%orbitals=orbitals
!  others:
   do imesh=1,nmesh
     call pawrad_copy(radmesh(imesh),paw_atdata%radmesh(imesh))
   end do
   paw_atdata%ncore=ncore
   paw_atdata%tncore=tncore
   paw_atdata%tnvale0=tnvale0
   paw_atdata%tproj=tproj
   paw_atdata%vlocr=vlocr

   if(pawtab%shape_type==-1) then
     ABI_ALLOCATE(paw_atdata%shpf,(shpf_mesh%mesh_size,pawtab%l_size))
     paw_atdata%shpf=shpf
   end if


 end subroutine fill_paw_atdata
!!***


end subroutine psp7in
!!***
