!{\src2tex{textfont=tt}}
!!****f* ABINIT/wvlpaw_sin2gauss
!! NAME
!!  wvlpaw_sin2gauss
!!
!! FUNCTION
!!  Converts a f(x)=sum_i^N_i a_i sin(b_i x)+ c_i cos( d_i x) to
!!    f(x)=sum_j e_j exp(f_j x), where e and f are complex numbers.
!!
!! COPYRIGHT
!!  Copyright (C) 2011-2013 ABINIT group (T. Rangel)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!! basis_size =  size of the lmn basis
!! mparam = number of terms in the summatory (N_i, see the expression above)
!! nparam = Array containing the parameters (a_i, b_i,c_i,d_i)
!! wvl = wavelets data type
!!
!! OUTPUT
!!
!! SIDE EFFECTS
!! On output wvl%pfac and wvl%parg are filled with complex parameters (e_i, f_i)
!!
!! NOTES
!!
!! PARENTS
!!      pspatm_abinit
!!
!! CHILDREN
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

#include "abi_common.h"

 subroutine wvlpaw_sin2gauss(basis_size,mparam,nparam,&
&param,wvl)

 use defs_basis
 use m_profiling
 use m_pawtab, only : wvlpaw_type

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'wvlpaw_sin2gauss'
!End of the abilint section

  implicit none
  !
!Arguments ------------------------------------
  integer,intent(in) :: mparam,basis_size
  integer,intent(in) :: nparam(basis_size)
  real(dp),intent(in) :: param(mparam,basis_size)
  type(wvlpaw_type),intent(inout):: wvl

!Local variables ------------------------------
  integer :: i,ii,ib,ngauss,nterm
  real(dp) :: sep
  real(dp) :: a1(mparam),a2(mparam),a3(mparam),a4(mparam),a5(mparam)
  real(dp) :: b1r(mparam),b2r(mparam),b1i(mparam),b2i(mparam)
  !
  !extra variables, use to debug
  !
  !integer::igau,nr,unitp
  !real(dp)::step,rmax
  !real(dp),allocatable::r(:), y(:)
  !complex::fac,arg
  !complex(dp),allocatable::f(:)
!************************************************************************

!
!
!Convert from \sum(sin+cos) expressions to sums of complex gaussians
!(only works for option=4, see fit_gen)
!
!get number of coefficients:
 ii=0
 do ib=1,basis_size
   nterm=nparam(ib)/4  !option=4, there are 4 parameters for each term
   ii=ii+nterm*2 !two gaussians for each term
 end do
!
!Allocate objects
!
 ngauss=ii
 wvl%ptotgau=ngauss !total number of complex gaussians
 ABI_ALLOCATE(wvl%pfac,(2,ngauss))
 ABI_ALLOCATE(wvl%parg,(2,ngauss))
 ABI_ALLOCATE(wvl%pngau,(basis_size))
 wvl%pngau(1:basis_size)=nparam(1:basis_size)/2 !option=4
!
 ii=0
 do ib=1,basis_size
!  
!  Get parameters in sin+cos expansion:
!  Option4: \sum a1 exp(-a2 x^2) ( a3 sin(k x^2) + a4 cos(k x^2))
!  
   nterm=nparam(ib)/4  !option=4
!  
   a1(1:nterm)=param(1:nterm,ib)
   a2(1:nterm)=param(nterm+1:nterm*2,ib)
   a3(1:nterm)=param(nterm*2+1:nterm*3,ib)
   a4(1:nterm)=param(nterm*3+1:nterm*4,ib)
   sep=1.1d0
   do i=1,nterm
     a5(i)=sep**(i)
   end do
!  
!  Now translate them to a sum of complex gaussians: 
!  pngau(ib)=nterm*2  
!  Two gaussians by term:
!  
!  First gaussian
   b1r(1:nterm)= a1(1:nterm)*a4(1:nterm)/2.d0 !coefficient, real
   b1i(1:nterm)=-a1(1:nterm)*a3(1:nterm)/2.d0 !coefficient, imag
   b2r(1:nterm)=-a2(1:nterm)   !exponential, real
   b2i(1:nterm)= a5(1:nterm)   !exponential, imag
!  
   wvl%pfac(1,ii+1:ii+nterm)=b1r(1:nterm)
   wvl%pfac(2,ii+1:ii+nterm)=b1i(1:nterm)
   wvl%parg(1,ii+1:ii+nterm)=b2r(1:nterm) 
   wvl%parg(2,ii+1:ii+nterm)=b2i(1:nterm) 
!  Second gaussian
   wvl%pfac(1,ii+nterm+1:ii+nterm*2)= b1r(1:nterm)
   wvl%pfac(2,ii+nterm+1:ii+nterm*2)=-b1i(1:nterm)
   wvl%parg(1,ii+nterm+1:ii+nterm*2)= b2r(1:nterm) 
   wvl%parg(2,ii+nterm+1:ii+nterm*2)=-b2i(1:nterm) 
!  
   ii=ii+nterm*2
 end do
!
!begin debug
!
!write(*,*)'wvlpaw_sin2gauss, comment me'
!nr=3000
!rmax=10.d0
!ABI_ALLOCATE(r,(nr))
!ABI_ALLOCATE(f,(nr))
!ABI_ALLOCATE(y,(nr))
!step=rmax/real(nr-1,dp)
!do ir=1,nr
!r(ir)=real(ir-1,dp)*step
!end do
!!
!ii=0
!do ib=1,basis_size
!unitp=500+ib
!f(:)=czero
!!
!do igau=1,wvl%pngau(ib)
!ii=ii+1
!arg=cmplx(wvl%parg(1,ii),wvl%parg(2,ii))
!fac=cmplx(wvl%pfac(1,ii),wvl%pfac(2,ii))
!f(:)=f(:)+fac*exp(arg*r(:)**2)
!end do
!do ir=1,nr
!write(unitp,'(3f16.7)')r(ir),real(f(ir))!,y(ir)
!end do
!end do
!ABI_DEALLOCATE(r)
!ABI_DEALLOCATE(f)
!ABI_DEALLOCATE(y)
!
!end debug
!

 end subroutine wvlpaw_sin2gauss
!!***


